/*
 * Project: irond
 * Package: src.de.fhhannover.inform.ifmap.datastructure
 * File:    Link2.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.ifmap.datastructure;

import de.fhhannover.inform.ifmap.exception.LinkConstructionException;

/**
 * Represents a Link
 * @author Tobias
 *
 */
public class Link2 extends IfmapDataType{

	private String name;
	private Identifier2 mIdentifier1;	
	private Identifier2 mIdentifier2;	

	public Link2(String name, Identifier2 i1, Identifier2 i2) 
	throws LinkConstructionException{
		super();
		if(i1 == null || i2 == null){
			throw new LinkConstructionException("One or two Identifiers are null!");
		}
		this.name = name;
		this.mIdentifier1 = i1;
		this.mIdentifier2 = i2;		
	}
	
	public String getName(){
		return name;
	}
	
	public Identifier2 getIdentifier1(){
		return mIdentifier1;
	}
	
	public Identifier2 getIdentifier2(){
		return mIdentifier2;
	}

	@Override
	public boolean equals(Object o) {
		if(o == null) return false;
		if(this == o) return true;
		if(!(o instanceof Link2)) return false;
		
		Link2 l2 = (Link2)o;
		
		if(this.hashCode() == l2.hashCode()){
			return true;
		}
		else return false;
	}

	@Override
	public int hashCode() {
		StringBuffer buf = new StringBuffer();
		buf.append(this.getName());
		buf.append(this.mIdentifier1.hashCode()+this.mIdentifier2.hashCode());		
		//buf.append(this.getMetadata().toString());
		return buf.toString().hashCode();
	}
	
	@Override
	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append(this.getName()+" [");
		buf.append(this.mIdentifier1.getName()+", "+ this.mIdentifier2.getName()+
				" ] {"+this.hashCode()+"}\r\n     |--[M] "+this.getMetadata().toString());
		return buf.toString();
	}	
}
