/*
 * Project: irond
 * Package: src.de.fhhannover.inform.ifmap.datastructure
 * File:    Metadata.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.ifmap.datastructure;

import java.util.Enumeration;
import java.util.Hashtable;

public class Metadata {

	private String mName;
	private String mPublisher;
	private String mCardinality;
	private String mTimestamp;
	private Hashtable<String, String> mAttributes;

	public Metadata() {
		mAttributes = new Hashtable<String, String>();
	}
	public Metadata(String name) {
		this();
		mName = name;
	}

	public Metadata(String name, String publisher, String cardinality, String timestamp){
		mName = name;
		mPublisher = publisher;
		mCardinality = cardinality;
		mTimestamp = timestamp;
	}
	
	public Metadata(String name, String publisher, String cardinality, String timestamp, 
			Hashtable<String, String> attributes){
		mName = name;
		mPublisher = publisher;
		mCardinality = cardinality;
		mTimestamp = timestamp;
		if(attributes != null){
			mAttributes = attributes;
		}
		else {
			mAttributes = new Hashtable<String, String>();
		}
	}
	
	public boolean hasAttributes(){
		return !mAttributes.isEmpty();
	}
	
	public void addAttribute(String key, String value){
		mAttributes.put(key, value);
	}
	
	public String getAttribute(String key){
		return mAttributes.get(key);
	}
	
	public Hashtable<String, String> getAttributes(){
		Hashtable<String, String> attr = new Hashtable<String, String>();
		Enumeration<String> keys = mAttributes.keys();
		while(keys.hasMoreElements()){
			String key = keys.nextElement();
			attr.put(key, mAttributes.get(key));
		}
		return attr;
	}
	
	public Metadata copy() {			
		return new Metadata(mName, mPublisher, mCardinality, mTimestamp, getAttributes());
	}

	public String getName() {
		return mName;
	}

	public void setName(String mName) {
		this.mName = mName;
	}

	public String getPublisher() {
		return mPublisher;
	}

	public void setPublisher(String publisher) {
		mPublisher = publisher;
	}

	public String getCardinality() {
		return mCardinality;
	}

	public void setCardinality(String cardinality) {
		mCardinality = cardinality;
	}

	public boolean isSingleValue(){
		return mCardinality.equalsIgnoreCase("singleValue");
	}
	
	public String getTimestamp() {
		return mTimestamp;
	}

	public void setTimestamp(String timestamp) {
		mTimestamp = timestamp;
	}
	
	@Override
	public boolean equals(Object o) {
		if(o == null) return false;
		if(this == o) return true;
		if(!(o instanceof Metadata)) return false;
		
		Metadata meta = (Metadata)o;
		
		if(this.hashCode() == meta.hashCode()){
			return true;
		}
		else return false;
	}
	
	@Override
	public int hashCode(){
		StringBuffer buf = new StringBuffer();
		buf.append(this.mName);
		buf.append(this.mPublisher);
		buf.append(this.mCardinality);
		//buf.append(this.mTimestamp);
		buf.append(mAttributes.toString());
		return buf.toString().hashCode();
	}	
	
	@Override
	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append("name="+this.mName+", publisher="+this.mPublisher+", timestamp="+this.mTimestamp+" ,cardinality="+
				this.mCardinality+" [ "+this.mAttributes.toString()+" ] {"+this.hashCode()+"}");
		return buf.toString();
	}
}
