/*
 * Project: irond
 * Package: src.de.fhhannover.inform.ifmap.properties
 * File:    PropertiesReader.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.ifmap.properties;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;

/**
 * Reads properties from a file
 * @author Tobias
 *
 */
public class PropertiesReader {
	private static Properties props;
	private static boolean flag = false;
	private static String path = "ifmap.properties";
	
	/**
	 * Sets the path to the properties file.
	 */
	public static void setPath(String path){
		PropertiesReader.path = path;		
	}
	
	/**
	 * Loads the properties to memory
	 * @param path the property file
	 */
	public static void loadProperties(String path)
	throws FileNotFoundException, IOException{
		props = new Properties();
		FileInputStream in;		
		in = new FileInputStream(path);
		props.load(in);
		in.close();
		initConfig(props);
		flag = true;			
	}
	
	private static void initConfig(Properties conf){			
			Config.MAPSERVER_URL = conf.getProperty("mapserver.url", "localhost:8443");
			Config.MAPSERVER_KEYSTORE_PATH = conf.getProperty("mapserver.keystore.path", ".keystore");		
			Config.MAPSERVER_KEYSTORE_PASSWORD = conf.getProperty("mapserver.keystore.password", "mapdhcp2010");
			Config.MAPSERVER_TRUSTSTORE_PATH = conf.getProperty("mapserver.truststore.path", ".truststore");
			Config.MAPSERVER_TRUSTSTORE_PASSWORD = conf.getProperty("mapserver.truststore.password", "mapdhcp2010");
			Config.MAPSERVER_BASICAUTH_ENABLED = Boolean.parseBoolean(conf.getProperty("mapserver.basicauth.enabled", "false"));
			Config.MAPSERVER_BASICAUTH_USER = conf.getProperty("mapserver.basicauth.user", "test");			
			Config.MAPSERVER_BASICAUTH_PASSWORD = conf.getProperty("mapserver.basicauth.password", "test");				
	}
	
	/**
	 * Gets a specific property
	 * @param ident The property name
	 * @return the value of a property
	 * @throws IOException 
	 * @throws FileNotFoundException 
	 */
	public static String getProperty(String ident) throws FileNotFoundException, IOException{
		if(!flag){
			loadProperties(PropertiesReader.path);
		}
		String value = props.getProperty(ident);
		return value;
	}
}
