/*
 * Project: irond
 * Package: src.de.fhhannover.inform.ifmap.transform
 * File:    MessageTransformer.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.ifmap.transform;

/**
 * Transform JAXB-RI Objects to JAVA-Objects
 */
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.UUID;

import org.apache.xerces.dom.ElementNSImpl;
import org.trustedcomputinggroup._2010.ifmap._2.AccessRequestType;
import org.trustedcomputinggroup._2010.ifmap._2.DeleteResultType;
import org.trustedcomputinggroup._2010.ifmap._2.DeviceType;
import org.trustedcomputinggroup._2010.ifmap._2.ErrorResultType;
import org.trustedcomputinggroup._2010.ifmap._2.IPAddressType;
import org.trustedcomputinggroup._2010.ifmap._2.IdentityType;
import org.trustedcomputinggroup._2010.ifmap._2.MACAddressType;
import org.trustedcomputinggroup._2010.ifmap._2.MetadataListType;
import org.trustedcomputinggroup._2010.ifmap._2.ObjectFactory;
import org.trustedcomputinggroup._2010.ifmap._2.PollResultType;
import org.trustedcomputinggroup._2010.ifmap._2.ResultItemType;
import org.trustedcomputinggroup._2010.ifmap._2.SearchResultType;
import org.trustedcomputinggroup._2010.ifmap._2.SubscribeRequestType;
import org.trustedcomputinggroup._2010.ifmap._2.UpdateResultType;
import org.trustedcomputinggroup._2010.ifmap._2.SubscribeRequestType.Update;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import de.fhhannover.inform.ifmap.communication.SOAPConnector;
import de.fhhannover.inform.ifmap.datastructure.Identifier;
import de.fhhannover.inform.ifmap.datastructure.Identifier2;
import de.fhhannover.inform.ifmap.datastructure.IdentifierContainer;
import de.fhhannover.inform.ifmap.datastructure.IfmapDataType;
import de.fhhannover.inform.ifmap.datastructure.Link;
import de.fhhannover.inform.ifmap.datastructure.Link2;
import de.fhhannover.inform.ifmap.datastructure.LinkContainer;
import de.fhhannover.inform.ifmap.datastructure.Metadata;
import de.fhhannover.inform.ifmap.datastructure.MetadataRepository;
import de.fhhannover.inform.ifmap.event.IdentifierChangedReceiver;
import de.fhhannover.inform.ifmap.event.StatusChangedReceiver;
import de.fhhannover.inform.ifmap.exception.LinkConstructionException;
import de.fhhannover.inform.ifmap.exception.NoValidIdentifierTypeException;
import de.fhhannover.inform.ifmap.util.Crypto;

public class MessageTransformer {
	private ObjectFactory ifmap_factory = new ObjectFactory();
	private IdentifierContainer search_identifier = new IdentifierContainer();
	private ArrayList<StatusChangedReceiver> receiver_status = new ArrayList<StatusChangedReceiver>();
	private ArrayList<IdentifierChangedReceiver> receiver_identifier = new ArrayList<IdentifierChangedReceiver>();
	private HashMap<String, Identifier> subscribtions = new HashMap<String, Identifier>();
	private Hashtable<String, Link> str2link = new Hashtable<String, Link>();
	private MetadataRepository mMetadataRepository = new MetadataRepository();
	private SOAPConnector soap;

	private Poller poller = new Poller();
	private boolean isDone = false;

	public MessageTransformer() {
		soap = new SOAPConnector(this);
	}

	public boolean connect() throws RemoteException {
		return soap.connect();
	}

	public void disconnect() {
		soap.disconnect();
	}

	public String getPublisherID() {
		return soap.getPublisherID();
	}

	public String getSessionID() {
		return soap.getSessionID();
	}

	public void poll() throws RemoteException {
		ArrayList<IfmapDataType> ifmapResult = null;
		ArrayList<IfmapDataType> rNew = null;
		ArrayList<IfmapDataType> rUpdate = null;
		ArrayList<IfmapDataType> rDelete = null;
		PollResultType result = soap.sendPoll();		
		if (result != null) {
			ifmapResult = new ArrayList<IfmapDataType>();
			List<Object> list = result.getSearchResultOrUpdateResultOrDeleteResult();
			for (Object jaxb : list) {				
				if (jaxb instanceof ErrorResultType) {
					continue;
				}
								
				List<ResultItemType> rit;
				String rName;
				//TODO NotifyResultType
				if (jaxb instanceof UpdateResultType) {
					if(rNew == null){
						rNew = new ArrayList<IfmapDataType>();
					}
					UpdateResultType srt = (UpdateResultType) jaxb;
					rit = srt.getResultItem();
					rName = "updateResult";
					ArrayList<IfmapDataType> pc =  processSearchUpdateDeleteResult(rit);
					for(IfmapDataType arr : pc){
						if(!rNew.contains(arr))
							rNew.add(arr);
						//this.mMetadataRepository.addEntity(arr);
					}					
				} else if (jaxb instanceof SearchResultType) {
					if(rUpdate == null){
						rUpdate = new ArrayList<IfmapDataType>();
					}
					SearchResultType srt = (SearchResultType) jaxb;
					rit = srt.getResultItem();
					rName = "searchResult";
					ArrayList<IfmapDataType> pc =  processSearchUpdateDeleteResult(rit);
					for(IfmapDataType arr : pc){
						if(!rUpdate.contains(arr))
							rUpdate.add(arr);
						//this.mMetadataRepository.addEntity(arr);
					}
					
				} else if (jaxb instanceof DeleteResultType) {
					if(rDelete == null){
						rDelete = new ArrayList<IfmapDataType>();
					}
					DeleteResultType srt = (DeleteResultType) jaxb;
					rit = srt.getResultItem();
					rName = "deleteResult";
					ArrayList<IfmapDataType> pc =  processSearchUpdateDeleteResult(rit);
					for(IfmapDataType arr : pc){
						if(!rDelete.contains(arr))
							rDelete.add(arr);
						//this.mMetadataRepository.removeEntity(arr);
					}
				} else {
					continue;
				}							
			}
		}
		//System.err.println("Size:" +rUpdate.size());
		//mMetadataRepository.printMaps();
		notifyRepositoryChanged(rNew, rUpdate, rDelete);		
		//notifyReceiver(rName, ifmap);
	//	this.n
	}

	private void notifyReceiver(String name, IdentifierContainer conti) {
		// System.err.println(name + ": " + conti.toString());
		if (name.equals("searchResult")) {
			this.notifyIdentifierNew(conti);
		} else if (name.equals("updateResult") || name.equals("notifyResult")) {
			this.notifyIdentifierChanged(conti);
		} else if (name.equals("deleteResult")) {
			this.notifyIdentifierDelete(conti);
		}
	}
	private void notifyReceiver(String name, ArrayList<IfmapDataType> rNew, ArrayList<IfmapDataType> rUpdate,
			ArrayList<IfmapDataType> rDelete) {
		// System.err.println(name + ": " + conti.toString());
		/*for(IfmapDataType entity : list){
			if (name.equals("searchResult")) {
				this.mMetadataRepository.addEntity(entity);
			} else if (name.equals("updateResult") || name.equals("notifyResult")) {
				this.mMetadataRepository.addEntity(entity);
			} else if (name.equals("deleteResult")) {
				this.mMetadataRepository.removeEntity(entity);
			}
		}*/
	}

	public void enablePolling(boolean poll) {
		if (!poller.isAlive()) {
			if (poll) {
				poller = new Poller();
				poller.start();
			}
		} else {
			if (!poll) {
				isDone = true;
			}
		}

	}

	public boolean subscribeUpdate(Object identifier, long depth, long size,
			String filter, String links, String[] terminal)
			throws RemoteException {
		SubscribeRequestType srt = ifmap_factory.createSubscribeRequestType();
		Update update = ifmap_factory.createSubscribeRequestTypeUpdate();
		// System.err.println("subscribeUpdate");
		
		if (depth > 0l)
			update.setMaxDepth(depth);
		if (size > 0l)
			update.setMaxSize(size);
		if (filter != null && !filter.trim().isEmpty())
			update.setResultFilter(filter);
		if (links != null && !links.trim().isEmpty())
			update.setMatchLinks(links);

		if (terminal != null && terminal.length > 0) {
			StringBuffer sb = new StringBuffer();
			int i = 0;
			for (; i < terminal.length - 1; i++) {
				if (!terminal[i].trim().isEmpty()) {
					sb.append(terminal[i]);
					sb.append(',');
				}
			}
			if (!terminal[i].trim().isEmpty()) {
				sb.append(terminal[i]);
			}

			String terminalString = sb.toString();

			if (!terminalString.trim().isEmpty()) {
				update.setTerminalIdentifierType(terminalString);
			}
		}

		String _name = Crypto.randomUUID(16);

		if (identifier instanceof IPAddressType) {
			IPAddressType ip = (IPAddressType) identifier;
			update.setIpAddress(ip);
		} else if (identifier instanceof AccessRequestType) {
			AccessRequestType ar = (AccessRequestType) identifier;
			update.setAccessRequest(ar);
		} else if (identifier instanceof MACAddressType) {
			MACAddressType mac = (MACAddressType) identifier;
			update.setMacAddress(mac);
		} else if (identifier instanceof DeviceType) {
			DeviceType d = (DeviceType) identifier;
			update.setDevice(d);
		} else if (identifier instanceof IdentityType) {
			IdentityType id = (IdentityType) identifier;
			update.setIdentity(id);
		} else {
			System.err.println("well, no valid type found :(");
			return false;
		}

		update.setName(_name);

		if (!subscribtions.containsKey(_name)) {
			srt.getUpdateOrDelete().add(update);
			subscribtions.put(_name, null);
		}
		// System.err.println(srt.getUpdateOrDelete().toString());
		return soap.subscribeUpdate(srt);

	}

	public boolean subscribeUpdate(IdentifierContainer conti, long depth,
			long size, String filter, String links, String[] terminal)
			throws RemoteException {
		SubscribeRequestType srt = ifmap_factory.createSubscribeRequestType();
		for (Identifier ident : conti) {
			Update update = this.ifmap_factory
					.createSubscribeRequestTypeUpdate();

			// we calculate the name for the subscription
			String _name = ident.getName() + ident.getAttributes();
			// create a md5 hash and take every second character for a string
			// with length of 16
			String md5 = Crypto.generateMD5(_name);
			StringBuffer buffer = new StringBuffer();
			for (int i = 0; i < md5.length(); i = i + 2) {
				buffer.append(md5.charAt(i));
			}

			update.setName(buffer.toString());
			update.setMaxDepth(1l);
			Object o = ident.getRequestObject();
			if (o instanceof AccessRequestType) {
				update.setAccessRequest((AccessRequestType) o);
			} else if (o instanceof DeviceType) {
				update.setDevice((DeviceType) o);
			} else if (o instanceof IPAddressType) {
				IPAddressType ip = (IPAddressType) o;
				update.setIpAddress(ip);
			} else if (o instanceof IdentityType) {
				update.setIdentity((IdentityType) o);
			} else if (o instanceof MACAddressType) {
				update.setMacAddress((MACAddressType) o);
			} else {
				return false;
			}

			if (!subscribtions.containsKey(_name)) {
				srt.getUpdateOrDelete().add(update);
				subscribtions.put(_name, ident);
			}
		}
		return soap.subscribeUpdate(srt);
	}

	public IdentifierContainer getSearchIdentifier(String filter)
			throws RemoteException {
		search_identifier.clear();
		if (soap.connect()) {
			List<Object> list = soap.getDumpResult(filter);
			if (list == null) {
				return null;
			} else {
				for (Object o : list) {
					Identifier id = null;
					try {
						id = new Identifier(o);
					} catch (NoValidIdentifierTypeException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					search_identifier.add(id);
				}
			}
			return search_identifier;
		} else
			return null;
	}

	public long getLastUpdate() throws RemoteException {
		return soap.getLastUpdate();
	}

	public void addStatusListener(StatusChangedReceiver r) {
		if (!receiver_status.contains(r))
			receiver_status.add(r);
	}

	public void addUpdateListener(IdentifierChangedReceiver r) {
		if (!receiver_identifier.contains(r))
			receiver_identifier.add(r);
	}

	public synchronized void notifyIdentifierNew(IdentifierContainer conti) {
		for (IdentifierChangedReceiver r : receiver_identifier) {
			r.processNew(conti);
			// System.err.println(conti.toString());
		}
	}

	public synchronized void notifyIdentifierChanged(IdentifierContainer conti) {
		for (IdentifierChangedReceiver r : receiver_identifier) {
			r.processUpdate(conti);
		}
	}

	public synchronized void notifyIdentifierDelete(IdentifierContainer conti) {
		for (IdentifierChangedReceiver r : receiver_identifier) {
			r.processDelete(conti);
		}
	}
	
	public synchronized void notifyRepositoryChanged(ArrayList<IfmapDataType> newData, 
			ArrayList<IfmapDataType>updateData, ArrayList<IfmapDataType>deleteData) {
		for (IdentifierChangedReceiver r : receiver_identifier) {
			r.processAll(newData, updateData, deleteData);
		}
	}

	public synchronized void notifyStatusChangedReceiver(String evt) {
		for (StatusChangedReceiver r : receiver_status) {
			r.processStatusEvent(evt);
		}
	}

	private synchronized ArrayList<IfmapDataType> processSearchUpdateDeleteResult(List<ResultItemType> l) {
		ArrayList<IfmapDataType> list = null;
		if (l != null && !l.isEmpty()) {
			list = new ArrayList<IfmapDataType>();
			// System.err.println("PollResult-size:" +l.size());
			for (ResultItemType result : l) {
				processResultItem(result, list);
			}			
		}
		return list;
	}

	private void processResultItem(ResultItemType result, ArrayList<IfmapDataType> list) {
		MetadataListType meta = result.getMetadata();		
		//we are only interested in metadata-types
		if (meta != null) {			
			Identifier2 i1 = null;
			Identifier2 i2 = null;
			Object o1 = result.getAccessRequestOrIdentityOrIpAddress().get(0);			
			Object o2 = null;
			if (result.getAccessRequestOrIdentityOrIpAddress().size() == 2) {
				o2 = result.getAccessRequestOrIdentityOrIpAddress().get(1);
			}
			
			try {
				i1 = new Identifier2(o1);
				if(o2 != null){
					i2 = new Identifier2(o2);
				}
			} catch (NoValidIdentifierTypeException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			Metadata[] metaContainer = processMetadata(meta);
			if(metaContainer != null){
				if(i2 != null){					
					for(Metadata m : metaContainer){
						Link2 l = null;
						try {
							l = new Link2(m.getName(), i1, i2);							
						} catch (LinkConstructionException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
						if(l != null){
							l.addOrReplaceMetadata(m);
						}
						list.add(l);
					}					
					list.add(i2);
				}
				else {					
					for(Metadata m : metaContainer){
						i1.addOrReplaceMetadata(m);
					}					
				}
				list.add(i1);
			}				
		}
	}

	private Metadata[] processMetadata(MetadataListType metadata) {
		List<Object> ml = metadata.getAny();
		Metadata[] container = null;
		if(ml != null){
			container = new Metadata[ml.size()];
			// iterate over each child of ifmap:metadata
			for (int i = 0; i < ml.size(); i++) {
				ElementNSImpl meta = (ElementNSImpl) ml.get(i);
				Metadata data = new Metadata(meta.getNodeName());
				//Link link = new Link(meta.getNodeName(), i1, i2);
				NamedNodeMap map = meta.getAttributes();
				// get operational attributes
				for (int n = 0; n < map.getLength(); n++) {
					Node node = map.item(n);
					String name = node.getNodeName();
					if(name.equalsIgnoreCase("ifmap-publisher-id")){
						data.setPublisher(node.getNodeValue());
					}
					else if(name.equalsIgnoreCase("ifmap-timestamp")){
						data.setTimestamp(node.getNodeValue());					
					}
					else if(name.equalsIgnoreCase("ifmap-cardinality")){
						data.setCardinality(node.getNodeValue());					
					}
					else if(!name.startsWith("xmlns:")){
						data.addAttribute(name, node.getNodeValue());
					}
				}					
				
				// now process children of metadate element
				NodeList nl = meta.getChildNodes();
				
				// -1 because of trash ;)
				for (int j = 0; j < nl.getLength() - 1; j++) {
					// process the next child
					processChild(data, nl.item(j));
				}
				//FIXME What if nl has TextContent only? we need to process this too!
				container[i] = data;
			}			
		}
		// System.err.println(container.toString());
		return container;
	}

	/**
	 * 
	 * @param data object to save payload of metadata element
	 * @param n the current child to be processed
	 */
	private void processChild(Metadata data, Node n) {
		// add attributes
		NamedNodeMap nnm = n.getAttributes();
		if (nnm != null && nnm.getLength() > 0) {
			for (int l = 0; l < nnm.getLength(); l++) {
				String name = nnm.item(l).getNodeName();
				String value = nnm.item(l).getNodeValue();
				data.addAttribute(name, value);										
			}
		}				
		
		// add element and its content
		if(n.getTextContent() != null){
			data.addAttribute(n.getNodeName(), n.getTextContent());
		}
		
		// process children if necessary
		if (n.getChildNodes() != null && n.getChildNodes().getLength() > 0){
			for (int i = 0; i < n.getChildNodes().getLength() - 1; i++) {
				processChild(data, n.getChildNodes().item(i));
			}
		}
	}

	private class Poller extends Thread {
		public Poller() {
		}

		@Override
		public void run() {
			while (!isDone) {
				// notifyStatusChangedReceiver("Starte Polling...");
				try {
					poll();					
				} catch (RemoteException e) {
					notifyStatusChangedReceiver("Poll Error...");
					try {
						sleep(10000);
					} catch (InterruptedException e1) {
						e1.printStackTrace();
					}
				}
			}
		}
	}
}
