/*
 * Project: irond
 * Package: src.de.fhhannover.inform.ifmap.visual
 * File:    MyFrame.java
 *
 * Copyright (C) 2010 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.ifmap.visual;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Observable;
import java.util.Observer;

import javax.imageio.ImageIO;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;

import prefuse.visual.VisualItem;
import de.fhhannover.inform.ifmap.control.IfmapFacade;
import de.fhhannover.inform.ifmap.datastructure.Identifier;
import de.fhhannover.inform.ifmap.datastructure.Identifier2;
import de.fhhannover.inform.ifmap.datastructure.IdentifierContainer;
import de.fhhannover.inform.ifmap.datastructure.IfmapDataType;
import de.fhhannover.inform.ifmap.datastructure.Link;
import de.fhhannover.inform.ifmap.datastructure.Metadata;
import de.fhhannover.inform.ifmap.event.IdentifierChangedReceiver;
import de.fhhannover.inform.ifmap.event.NodeSelectedReceiver;
import de.fhhannover.inform.ifmap.event.PublisherColorChangedReceiver;
import de.fhhannover.inform.ifmap.event.StatusChangedReceiver;
import de.fhhannover.inform.ifmap.exception.PropertiesNotFoundException;
import de.fhhannover.inform.ifmap.output.JPrefuse;

/**
 * This code was edited or generated using CloudGarden's Jigloo SWT/Swing GUI
 * Builder, which is free for non-commercial use. If Jigloo is being used
 * commercially (ie, by a corporation, company or business for any purpose
 * whatever) then you should purchase a license for each developer using Jigloo.
 * Please visit www.cloudgarden.com for details. Use of Jigloo implies
 * acceptance of these licensing terms. A COMMERCIAL LICENSE HAS NOT BEEN
 * PURCHASED FOR THIS MACHINE, SO JIGLOO OR THIS CODE CANNOT BE USED LEGALLY FOR
 * ANY CORPORATE OR COMMERCIAL PURPOSE.
 */
@SuppressWarnings("serial")
public class MyFrame extends javax.swing.JFrame implements Observer, PublisherColorChangedReceiver, 
IdentifierChangedReceiver{

	{
		//Set Look & Feel
		try {
			javax.swing.UIManager.setLookAndFeel(javax.swing.UIManager.getSystemLookAndFeelClassName());
		} catch(Exception e) {
			e.printStackTrace();
		}
	}


	{
		//Set Look & Feel
		try {
		//	javax.swing.UIManager.setLookAndFeel("com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel");
		} catch(Exception e) {
			e.printStackTrace();
		}
	}


	int windowWidth = 1024;
	int windowHeight = 768;
	int btnWidth = 146;
	int btnHeight = 38;
	int sepWidth = 32;
	int sepHeight = 42;

	private IfmapFacade dump;
	private JLabel esukom;
	private JPanel jPanel4;
	private JPanel topPanel;
	private JButton publisherButton;
	private JSplitPane jSplitPane2;
	private JPrefuse mPrefuse = null;
	private PublisherPanel pPanel = new PublisherPanel();

	private class DumpHelper extends Thread implements StatusChangedReceiver,
			NodeSelectedReceiver {
		private boolean pause = false;
		boolean active = true;
		private long lastUpdate = 0;		
		

		public void pause(boolean b) {
			pause = b;
		}

		public boolean isPaused() {
			return pause;
		}

		public void processStatusEvent(String string) {
			statusMessageLabel.setText(string);
		}

		@Override
		public void run() {			
			dump = IfmapFacade.getInstance("ifmap.properties");
			dump.addStatusListener(this);

			if (dump != null) {				
				if (graphPanel.getComponentCount() == 0) {
					mPrefuse = (JPrefuse) dump.getPrefuseGraph();
					mPrefuse.setTable(jTable1, jLabel1);
					mPrefuse.addNodeSelectedListener(this);
					dump.addUpdatesListener(mPrefuse);
					graphPanel.add(mPrefuse, BorderLayout.CENTER);
					graphPanel.revalidate();
					mPrefuse.revalidate();
					mPrefuse.pan(mPrefuse.getParent().getWidth() / 2, mPrefuse
							.getParent().getHeight() / 2);
				}
			}	

			while (active) {				
				if (!pause) {
					long time = 0;
					try {
						time = dump.getLastUpdateTime();						
						if (dump.getPublisherID() != null
								&& dump.getSessionID() != null) {
							setTitle("Publisher-ID: " + dump.getPublisherID()
									+ " / Session-ID: " + dump.getSessionID());
						}
						if (time > lastUpdate) {
							lastUpdate = time;
							IdentifierContainer conti = null;
							conti = dump.getSearchIdentifier();
							if (conti != null && conti.size() > 0) {
								if(dump.subscribe(conti, 1l, 0l, null, null, null)){									
									dump.enablePolling(true);
								}								
							}
						}						

					} catch (PropertiesNotFoundException ex) {
						System.err.println(ex.getMessage());
						System.exit(ERROR);
					} catch (RemoteException ex) {
						System.err.println(ex.getMessage());
						//System.exit(ERROR);
					}
				}								
				
				try {
					//timestampLabel.setText("Identifier: "+prefuse.countIdentifier()+"  Links: "+prefuse.countLinks());
					Thread.sleep(5000);
				} catch (Exception err) {
					err.printStackTrace();
				}
			}

		}

		@Override
		public void nodeSelected(VisualItem item) {
			if (item.get("object") instanceof Identifier2) {
				Identifier2 ident = (Identifier2) item.get("object");
				jLabel1.setText("<html><strong>"
						+ ident.getName() + "</strong></html>");
			} else if (item.get("object") instanceof Metadata) {
				Metadata meta = (Metadata) item.get("object");
				jLabel1.setText("<html><strong>"
						+ meta.getName() + "</strong> [ <i>ifmap-publisher</i>: <strong>"+meta.getPublisher()
						+ "</strong>   <i>ifmap-timestamp</i>: <strong>"+meta.getTimestamp()
						+"</strong>   <i>ifmap-cardinality</i>: <strong>"
						+meta.getCardinality()+"</strong> ]</html>");
			}

		}
	};

	private JToggleButton jButton1;
	private JButton saveButton;
	private JLabel timestampLabel;
	private JLabel statusMessageLabel;
	private JPanel statusPanel;
	private JScrollPane jScrollPane1;
	private JLabel logo_tnc;
	private JPanel jPanel1;
	private JLabel logo;
	private JButton jButton2;
	private JToggleButton jToggleButton1;
	private JLabel jLabel1;
	private JPanel jPanel3;
	private JPanel jPanel2;
	private JTable jTable1;
	private JPanel graphPanel;
	private DumpHelper dh;
	private SubscribeObservable subscription_gui = new SubscribeObservable();
	private MyFrame myFrame = this;

	public void startIfmap() {
		dh.pause(true);
		dh.start();
	}

	/**
	 * Auto-generated main method to display this JFrame
	 */
	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				MyFrame inst = new MyFrame();
				inst.setLocationRelativeTo(null);
				inst.setVisible(true);
				inst.startIfmap();
			}
		});
	}

	public MyFrame() {
		super();
		initGUI();
		this.subscription_gui.addObserver(this);
		dh = new DumpHelper();
		this.jScrollPane1.getViewport().setBackground(Color.WHITE);
		this.setTitle("IRONGUI 0.2.0");
	}

	private void initGUI() {
		try {
			this.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
			getContentPane().setBackground(new java.awt.Color(255, 255, 255));
			BorderLayout thisLayout = new BorderLayout();
			this.setFocusable(false);
			getContentPane().setLayout(thisLayout);
			{
				topPanel = new JPanel();
				BorderLayout topPanelLayout = new BorderLayout();
				topPanel.setLayout(topPanelLayout);
				getContentPane().add(topPanel, BorderLayout.NORTH);
				{
					jPanel4 = new JPanel();
					topPanel.add(jPanel4, BorderLayout.WEST);
					FlowLayout jPanel4Layout = new FlowLayout();
					jPanel4Layout.setAlignment(FlowLayout.LEFT);
					jPanel4.setLayout(jPanel4Layout);
					{
						jButton1 = new JToggleButton();
						jPanel4.add(jButton1);
						//jToolBar1.addSeparator(new Dimension(sepWidth, sepHeight));
						jButton1.setText("Start/Stop");
						jButton1.setFocusPainted(false);
						jButton1.setFocusable(false);
						jButton1.setIcon(new ImageIcon(getClass().getClassLoader().getResource("bullet_add.png")));
						jButton1.setToolTipText("Starts / stops dump-requests");
						jButton1.setRequestFocusEnabled(false);
						jButton1.setSelectedIcon(new ImageIcon(getClass().getClassLoader().getResource("bullet_delete.png")));
						jButton1.setRolloverIcon(new ImageIcon(getClass().getClassLoader().getResource("bullet_add.png")));
						jButton1.setRolloverSelectedIcon(new ImageIcon(getClass().getClassLoader().getResource("bullet_delete.png")));
						jButton1.addActionListener(new ActionListener() {
							public void actionPerformed(ActionEvent evt) {
								if(!dh.isAlive()){
									dh.start();
								}
								if (dh.isPaused())
									dh.pause(false);
								else{
									dh.pause(true);								
								}
							}
						});
					}
					{
						jToggleButton1 = new JToggleButton();
						jPanel4.add(jToggleButton1);
						//jToolBar1.addSeparator(new Dimension(sepWidth, sepHeight));
						jToggleButton1.setText("Animation On/Off");
						jToggleButton1.setSelected(true);
						jToggleButton1.setFocusable(false);
						jToggleButton1.setIcon(new ImageIcon(getClass().getClassLoader().getResource("mouse_off.png")));
						jToggleButton1.setToolTipText("Enable/disable animation");
						jToggleButton1.setSelectedIcon(new ImageIcon(getClass().getClassLoader().getResource("mouse_on.png")));
						jToggleButton1.addActionListener(new ActionListener() {
							public void actionPerformed(ActionEvent evt) {
								animationButtonActionPerformed(evt);
							}
						});
					}
					{
						saveButton = new JButton();
						jPanel4.add(saveButton);
						//jToolBar1.addSeparator();
						saveButton.setText("Snapshot");
						saveButton.setFocusable(false);
						saveButton.setIcon(new ImageIcon(getClass().getClassLoader().getResource("file_photo.png")));
						saveButton.setToolTipText("Save the actual view as PNG");
						saveButton.addActionListener(new ActionListener() {
							public void actionPerformed(ActionEvent evt) {
								saveButtonActionPerformed(evt);
							}
						});
					}
					{
						jButton2 = new JButton();
						jPanel4.add(jButton2);
						jButton2.setText("Subscribe");
						jButton2.setIcon(new ImageIcon(getClass().getClassLoader().getResource("rss.png")));
						jButton2.setFocusable(false);
						jButton2.setToolTipText("Subscribe to specific Metadata");
						jButton2.addActionListener(new ActionListener() {
							public void actionPerformed(ActionEvent evt) {
								if(subscription_gui == null)
									subscription_gui = new SubscribeObservable();
								
								subscription_gui.center();
								subscription_gui.showGUI();
							}
						});
					}
					{
						publisherButton = new JButton();
						jPanel4.add(publisherButton);
						publisherButton.setText("Publisher");
						publisherButton.setIcon(new ImageIcon(getClass().getClassLoader().getResource("user_info.png")));
						publisherButton.setFocusable(false);
						publisherButton.setToolTipText("Show publishers and their color associations");
						pPanel.registerPublisherColorChangedReceiver(myFrame);
						
						publisherButton.addActionListener(new ActionListener() {
							public void actionPerformed(ActionEvent evt) {
								if(mPrefuse != null){
									String[] publisher = mPrefuse.getPublishers();								
									if(publisher != null){									
										pPanel.addPublisher(publisher);																		
									}
									//	else {
										//	JOptionPane.showMessageDialog(getParent(), "No publishers yet.");
										//	}
									pPanel.pack();
									pPanel.setLocationRelativeTo(null);
									pPanel.setVisible(true);
								}
							}
						});
					}
				}
				{
					jPanel1 = new JPanel();
					topPanel.add(jPanel1, BorderLayout.EAST);
					BorderLayout jPanel1Layout = new BorderLayout();
					jPanel1.setLayout(jPanel1Layout);
					jPanel1.setOpaque(false);
					int tmpWidth = windowWidth - 3 * btnWidth - 3 * sepWidth;
					{
						logo_tnc = new JLabel();
						logo_tnc.setLayout(null);
						jPanel1.add(logo_tnc, BorderLayout.WEST);
						logo_tnc.setIcon(new ImageIcon(getClass().getClassLoader().getResource("logo_tnc.png")));
						logo_tnc.setToolTipText("Visit the Trust@FHH Website!");
						logo_tnc.addMouseListener(new MouseAdapter() {
							public void mouseExited(MouseEvent evt) {
								logo_tnc.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
							}
							public void mouseEntered(MouseEvent evt) {
								logo_tnc.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
							}
							public void mouseClicked(MouseEvent evt) {
								de.fhhannover.inform.ifmap.util.AppLauncher.openWebpageInBrowser("http://trust.inform.fh-hannover.de");
							}
						});
					}
					{
						logo = new JLabel();
						logo.setLayout(null);
						jPanel1.add(logo, BorderLayout.CENTER);
						logo.setIcon(new ImageIcon(getClass().getClassLoader().getResource("logo_iron.png")));
						logo.setToolTipText("More interesting IFMAP-stuff");
						logo.addMouseListener(new MouseAdapter() {
							public void mouseExited(MouseEvent evt) {
								logo.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
							}
							public void mouseEntered(MouseEvent evt) {
								logo.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
							}
							public void mouseClicked(MouseEvent evt) {
								de.fhhannover.inform.ifmap.util.AppLauncher.openWebpageInBrowser("http://trust.inform.fh-hannover.de/joomla/index.php/projects/iron");
							}
						});
					}
					{
						esukom = new JLabel();
						jPanel1.add(esukom, BorderLayout.EAST);
						esukom.setIcon(new ImageIcon(getClass().getClassLoader().getResource("esukom.png")));
						esukom.setToolTipText("ESUKOM Project-Homepage");
						esukom.addMouseListener(new MouseAdapter() {
							public void mouseExited(MouseEvent evt) {
								logo.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
							}
							public void mouseEntered(MouseEvent evt) {
								logo.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
							}
							public void mouseClicked(MouseEvent evt) {
								de.fhhannover.inform.ifmap.util.AppLauncher.openWebpageInBrowser("http://www.esukom.de/");
							}
						});
					}
				}
			}
			{
				jSplitPane2 = new JSplitPane();
				getContentPane().add(jSplitPane2, BorderLayout.CENTER);
				jSplitPane2.setDividerSize(3);
				jSplitPane2.setDividerLocation(500);
				jSplitPane2.setOrientation(JSplitPane.VERTICAL_SPLIT);
				{
					graphPanel = new JPanel();
					jSplitPane2.add(graphPanel, JSplitPane.TOP);
					BorderLayout graphPanelLayout = new BorderLayout();
					graphPanel.setLayout(graphPanelLayout);
					graphPanel.setBackground(new java.awt.Color(255, 255, 255));
					graphPanel.setPreferredSize(new java.awt.Dimension(1012, 468));
				}
				{
					jPanel2 = new JPanel();
					jSplitPane2.add(jPanel2, JSplitPane.BOTTOM);
					BorderLayout jPanel2Layout = new BorderLayout();
					jPanel2.setLayout(jPanel2Layout);
					jPanel2.setPreferredSize(new java.awt.Dimension(1012, 147));
					{
						jScrollPane1 = new JScrollPane();
						jPanel2.add(jScrollPane1, BorderLayout.CENTER);
						jScrollPane1.setPreferredSize(new java.awt.Dimension(1012, -214));
						jScrollPane1.setBackground(new java.awt.Color(255, 255,
								255));
						{
							jTable1 = new JTable();
							jScrollPane1.setViewportView(jTable1);
							jTable1.setCellSelectionEnabled(true);
							jTable1.setBackground(new java.awt.Color(255, 255,
									255));
							jTable1
							.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
							jTable1.setAutoCreateRowSorter(true);
						}
					}
					{
						jPanel3 = new JPanel();
						jPanel2.add(jPanel3, BorderLayout.NORTH);
						FlowLayout jPanel3Layout = new FlowLayout();
						jPanel3
						.setPreferredSize(new java.awt.Dimension(390,
								22));
						jPanel3.setLayout(jPanel3Layout);
						jPanel3Layout.setAlignment(FlowLayout.LEFT);
						jPanel3Layout.setAlignment(FlowLayout.LEFT);
						{
							jLabel1 = new JLabel();
							jPanel3.add(jLabel1);
							jLabel1.setHorizontalAlignment(SwingConstants.LEFT);
							jLabel1
							.setHorizontalTextPosition(SwingConstants.LEFT);
						}
					}
				}
			}
			{
				statusPanel = new JPanel();
				BorderLayout statusPanelLayout = new BorderLayout();
				getContentPane().add(statusPanel, BorderLayout.SOUTH);
				statusPanel.setPreferredSize(new java.awt.Dimension(777, 18));
				statusPanel.setLayout(statusPanelLayout);
				{
					statusMessageLabel = new JLabel();
					statusPanel.add(statusMessageLabel, BorderLayout.CENTER);
					statusMessageLabel.setPreferredSize(new java.awt.Dimension(
							645, 18));
					statusMessageLabel
							.setHorizontalAlignment(SwingConstants.RIGHT);
				}
				{
					timestampLabel = new JLabel();
					statusPanel.add(timestampLabel, BorderLayout.WEST);
					timestampLabel.setPreferredSize(new java.awt.Dimension(249,
							18));
				}
			}
			pack();
		} catch (Exception e) {
			// add your error handling code here
			e.printStackTrace();
		}
	}

	private void saveButtonActionPerformed(ActionEvent evt) {
		final JFileChooser fc = new JFileChooser();
		int width = this.graphPanel.getWidth();
		int height = this.graphPanel.getHeight();
		BufferedImage image = new BufferedImage(width, height,
				BufferedImage.TYPE_INT_RGB);
		Graphics2D g = image.createGraphics();
		this.graphPanel.paint(g);
		g.dispose();
		try {
			int returnVal = fc.showSaveDialog(this);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				ImageIO.write(image, "png", fc.getSelectedFile());
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	private void animationButtonActionPerformed(ActionEvent evt) {
		if (graphPanel.getComponentCount() > 0) {
			if (graphPanel.getComponent(0) instanceof JPrefuse) {
				JPrefuse prefuse = (JPrefuse) graphPanel.getComponent(0);
				prefuse.toggleAnimation();
			}
		}
	}

	@Override
	public void update(Observable arg0, Object arg1) {		
		SubscribeObservable so = (SubscribeObservable) arg0;
		try {
			if(dump.connect()){
				boolean b = dump.subscribe(so.getIdentifier(), so.getDepth(), so.getSize(), so.getFilter()
						, so.getLinks(), so.getTerminals());	
			if(b)
				{
					statusMessageLabel.setText("Subscribe erfolgreich!");					
					dump.enablePolling(true);
				}						
			else dh.processStatusEvent("Subscribe erfolglos!");
			}
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (PropertiesNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

	@Override
	public void setNewColorSelection(HashMap<String, Color> map, Color iColor, Color mColor) {
		this.mPrefuse.setNewMetadataColorAction(map, iColor, mColor);
	}

	@Override
	public void processAll(ArrayList<IfmapDataType> newData,
			ArrayList<IfmapDataType> updateData,
			ArrayList<IfmapDataType> deleteData) {		
		
	}

	@Override
	public void processNew(IdentifierContainer conti) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void processUpdate(IdentifierContainer conti) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void processDelete(IdentifierContainer conti) {
		// TODO Auto-generated method stub
		
	}

}
